// MIT © 2018 azu
"use strict";

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var execa = require("execa");
var pLimit = require("p-limit");
var IgnoreNodeManager = require("textlint-rule-helper").IgnoreNodeManager;

var limit = pLimit(1);
var report = function report(context) {
    var Syntax = context.Syntax,
        RuleError = context.RuleError,
        fixer = context.fixer,
        report = context.report;

    var filePath = context.getFilePath();
    var ignoreNodeManager = new IgnoreNodeManager();
    return _defineProperty({}, Syntax.Document, function (node) {
        if (!filePath) {
            return;
        }
        ignoreNodeManager.ignoreChildrenByTypes(node, [Syntax.CodeBlock, Syntax.Code, Syntax.Link, Syntax.Strong, Syntax.Emphasis, Syntax.BlockQuote, Syntax.Comment]);
        return limit(function () {
            return execa("proselint", ["--json", filePath]).catch(function (error) {
                if (error.code === "ENOENT") {
                    return report(node, new RuleError("Proselint not found.\n\nInstall proselint first:\n\n    pip install proselint\n\nMore information:\n\n    https://github.com/textlint-rule/textlint-rule-proselint \n"));
                }

                if (error.code !== 1) {
                    return;
                }
                var json = JSON.parse(error.stdout);
                if (json.status !== "success") {
                    return;
                }
                json.data.errors.forEach(function (lintError) {
                    var index = lintError.start - 1;
                    if (ignoreNodeManager.isIgnoredIndex(index)) {
                        return;
                    }
                    var canFixIt = typeof lintError.replacements === "string";
                    var source = lintError.source_url ? "Source: " + lintError.source_url : "";
                    var message = ("[" + lintError.check + "] " + lintError.message + "\n" + source + "\n").trim();
                    if (canFixIt) {
                        report(node, new RuleError(message, {
                            index: index,
                            severity: lintError.severity,
                            fix: fixer.replaceTextRange([index, lintError.end - 1], lintError.replacements)
                        }));
                    } else {
                        report(node, new RuleError(message, {
                            index: index,
                            severity: lintError.severity
                        }));
                    }
                });
            });
        });
    });
};
module.exports = {
    linter: report,
    fixer: report
};
//# sourceMappingURL=textlint-rule-proselint.js.map